import os
import shutil

from jinja2.environment import Environment
from jinja2.loaders import FileSystemLoader

def create_title_extension_dict(env: Environment) -> dict:
    ''' for a file "name.html". Dictionary k,v pair would be "name":"name.html" '''
    d = {x[:-5]:f'./{x}' for x in env.list_templates() if not x.startswith("_")}
    d.pop('base')
    return d

def default(template_dir, out_dir='./output'):
    try:
        os.mkdir(f'{out_dir}')
        os.mkdir(f'{out_dir}/style')
    except FileExistsError:
        shutil.rmtree(f'{out_dir}')
        os.mkdir(f'{out_dir}')
        os.mkdir(f'{out_dir}/style') 
    
    env = Environment(loader=FileSystemLoader(f'{template_dir}/html_templates'))
    filenames = [x for x in env.list_templates() if "base.html" not in x and not x.startswith("_")]
    nav_links = create_title_extension_dict(env)
    stylesheet_path = f'{template_dir}/css_templates'

    if os.path.exists(stylesheet_path):
        for x in os.listdir(stylesheet_path):
            shutil.copyfile(f'{stylesheet_path}/{x}', f'{out_dir}/style/{x}')
    
    for i in zip(filenames, nav_links):
        f = open(f'./output/{i[0]}', "+w")
        f.write(env.get_template(i[0]).render(title=i[1], nav_links=nav_links))
    



import click
import os
import zipfile

from http.server import HTTPServer, SimpleHTTPRequestHandler

import requests

from . import render_jinja

@click.group()
def main():
    pass

@click.command()
def download_templates():
    '''Downloads the example templates from Github'''

    r = requests.get('https://github.com/AirikWarren/Pyguin/archive/Templates.zip', stream=True)
    with open('templates.zip', 'wb') as fd:
     for chunk in r.iter_content(chunk_size=128):
             fd.write(chunk)
    
    with zipfile.ZipFile('templates.zip', 'r') as zip_ref:
        zip_ref.extractall()
    
    os.remove('templates.zip')

@click.command()
@click.argument('template_dir', type=click.Path(exists=True))
@click.option('--output', default='./output', help='Output director for static site')
def render_template(template_dir, output):
    '''Renders a Pyguin template into a static site'''
    render_jinja.default(template_dir, output)

@click.command()
@click.option('--bind', default="127.0.0.1")
@click.option('--port', default=8000)
def run(bind, port):
    '''starts a local development only server'''
    print(f'Starting a development server at http://{bind}:{port}')
    httpd = HTTPServer((bind, int(port)), SimpleHTTPRequestHandler)
    httpd.serve_forever()


main.add_command(download_templates)
main.add_command(render_template)
main.add_command(run)


import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pyguin",
    version="0.0.1",
    author="Airik Warren",
    author_email="airikwarren@gmail.com",
    description="A minimalistic static site generator intended to be very quick and intuitive to understand and extend",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/airikWarren/pyguin",
    project_urls={
        "Bug Tracker": "https://github.com/airikWarren/pyguin/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3", 
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    entry_points={
        'console_scripts': [
            'pyguin = pyguin.pyguin:main',
            ],
    },
    install_requires=['Jinja2', 'click', 'requests'],
    packages=setuptools.find_packages(),
    python_requires=">=3.6",
)
